<!doctype html>
<html class="no-js">
  <head>
    {%- block site_meta -%}
    <meta charset="utf-8"/>
    <meta name="viewport" content="width=device-width,initial-scale=1"/>
    <meta name="color-scheme" content="light dark">

    {%- if metatags %}{{ metatags }}{% endif -%}

    {%- block linktags %}
        {%- if hasdoc('about') -%}
          <link rel="author" title="{{ _('About these documents') }}" href="{{ pathto('about') }}" />
        {%- endif -%}
        {%- if hasdoc('genindex') -%}
          <link rel="index" title="{{ _('Index') }}" href="{{ pathto('genindex') }}" />
        {%- endif -%}
        {%- if hasdoc('search') -%}
          <link rel="search" title="{{ _('Search') }}" href="{{ pathto('search') }}" />
        {%- endif -%}
        {%- if hasdoc('copyright') -%}
          <link rel="copyright" title="{{ _('Copyright') }}" href="{{ pathto('copyright') }}" />
        {%- endif -%}
        {%- if next -%}
          <link rel="next" title="{{ next.title|striptags|e }}" href="{{ next.link|e }}" />
        {%- endif -%}
        {%- if prev -%}
          <link rel="prev" title="{{ prev.title|striptags|e }}" href="{{ prev.link|e }}" />
        {%- endif -%}
    {%- endblock linktags %}

    {# Favicon #}
    {%- if favicon -%}
      <link rel="shortcut icon" href="{{ pathto('_static/' + favicon, 1) }}"/>
    {%- endif -%}

    {#- Generator banner -#}
    <meta name="generator" content="sphinx-{{ sphinx_version }}, furo {{ furo_version }}"/>

    {%- endblock site_meta -%}

    {#- Site title -#}
    {%- block htmltitle -%}
      {% if pagename == master_doc %}
        <title>{{ _("Home") }} - {{ docstitle|striptags|e }}</title>
      {% else %}
        <title>{{ title|striptags|e }} - {{ docstitle|striptags|e }}</title>
      {% endif %}
    {%- endblock -%}

    {%- block styles -%}

    {#- Theme-related stylesheets -#}
    {%- block theme_styles -%}
    <link rel="stylesheet" href="{{ pathto(furo_assets['style'], 1) }}">
    <link rel="stylesheet" href="{{ pathto('_static/pygments.css', 1) }}">
    <link media="(prefers-color-scheme: dark)" rel="stylesheet" href="{{ pathto('_static/pygments_dark.css', 1) }}">
    {% include "partials/_head_css_variables.html" with context %}
    {%- endblock -%}

    {# Custom stylesheets #}
    {%- block regular_styles -%}
    {%- for path in css_files -%}
    {% if not path.filename.startswith("pygments_dark") -%}
      {{ css_tag(path) }}
    {%- endif %}
    {% endfor -%}
    {%- endblock regular_styles -%}

    {%- block extra_styles -%}
    <link rel="stylesheet" href="{{ pathto(furo_assets['furo-extensions.css'], 1) }}">
    {%- endblock -%}

    {%- endblock styles -%}

    {#- Custom front matter #}
    {%- block extrahead -%}{%- endblock -%}
  </head>
  <body dir="{{ direction }}">
    {% block body %}{% endblock %}

    {%- block scripts -%}

    {# Custom JS #}
    {%- block regular_scripts -%}
    {# This is *exactly* how `basic` handles this file. #}
    <script id="documentation_options" data-url_root="{{ pathto('', 1) }}" src="{{ pathto('_static/documentation_options.js', 1) }}"></script>
    {% for path in script_files -%}
      {{ js_tag(path) }}
    {% endfor -%}
    {%- endblock regular_scripts -%}

    {# Theme-related JavaScript code #}
    {%- block theme_scripts -%}
    <script src="{{ pathto(furo_assets['main.js'], 1) }}"></script>
    {%- endblock -%}

    {%- endblock scripts -%}
  </body>
</html>
